@@ -24,6 +24,7 @@
 #include "mail.h"
 #include "cpphelp.h"
 #include "windowmessages.h"
+#include "recipient.h"
 
 #include <gpgme++/context.h>
 #include <gpgme++/data.h>
@@ -36,11 +37,14 @@ typedef struct
   std::string name;
   std::string pgp_data;
   std::string cms_data;
+  std::string entry_id;
   HWND hwnd;
   shared_disp_t contact;
 } keyadder_args_t;
 
 static std::set <std::string> s_checked_entries;
+static std::vector <std::string> s_opened_contacts_vec;
+GPGRT_LOCK_DEFINE (s_opened_contacts_lock);
 
 static Addressbook::callback_args_t
 parse_output (const std::string &output)
@@ -207,6 +211,19 @@ open_keyadder (LPVOID arg)
                                   GpgME::Context::SpawnAllowSetFg |
                                   GpgME::Context::SpawnShowWindow));
   release_cArray (cargs);
+  if (!adder_args->entry_id.empty ())
+    {
+      log_dbg ("Removing entry from list of opened contacts.");
+      gpgrt_lock_lock (&s_opened_contacts_lock);
+      const auto id = adder_args->entry_id;
+      std::remove_if (s_opened_contacts_vec.begin(),
+                      s_opened_contacts_vec.end (),
+                      [id] (const auto &val) {
+                        return val == id;
+                      });
+      gpgrt_lock_unlock (&s_opened_contacts_lock);
+    }
+
   if (err)
     {
       log_error ("%s:%s: Err code: %i asString: %s",
@@ -295,6 +312,30 @@ Addressbook::edit_key_o (LPDISPATCH contact)
       TRACEPOINT;
       TRETURN;
     }
+  /* First check that we have not already opened an editor.
+     This can happen when starting the editor takes a while
+     and the user clicks multiple times because nothing
+     happens. */
+  const auto entry_id = get_oom_string_s (contact, "EntryID");
+
+  if (!entry_id.empty())
+    {
+      gpgrt_lock_lock (&s_opened_contacts_lock);
+      if (std::find (s_opened_contacts_vec.begin (),
+                     s_opened_contacts_vec.end (),
+                     entry_id) != s_opened_contacts_vec.end ())
+        {
+          log_dbg ("Contact already opened.");
+          /* TODO: Find the window if it exists and bring it to front. */
+          gpgrt_lock_unlock (&s_opened_contacts_lock);
+          TRETURN;
+        }
+      gpgrt_lock_unlock (&s_opened_contacts_lock);
+    }
+  else
+    {
+      log_dbg ("Empty EntryID.");
+    }
 
   auto user_props = MAKE_SHARED (get_oom_object (contact, "UserProperties"));
   if (!user_props)
@@ -332,6 +373,11 @@ Addressbook::edit_key_o (LPDISPATCH contact)
         }
     }
 
+  /* Insert into our vec of opened contacts. */
+  gpgrt_lock_lock (&s_opened_contacts_lock);
+  s_opened_contacts_vec.push_back (entry_id);
+  gpgrt_lock_unlock (&s_opened_contacts_lock);
+
   char *name = get_oom_string (contact, "Subject");
   if (!name)
     {
@@ -352,6 +398,7 @@ Addressbook::edit_key_o (LPDISPATCH contact)
   contact->AddRef ();
   memdbg_addRef (contact);
   args->contact = MAKE_SHARED (contact);
+  args->entry_id = entry_id;
 
   CloseHandle (CreateThread (NULL, 0, open_keyadder, (LPVOID) args, 0,
                              NULL));
@@ -394,7 +441,8 @@ Addressbook::check_o (Mail *mail)
                                                                     &err);
   for (const auto pair: recipient_entries)
     {
-      if (s_checked_entries.find (pair.first) != s_checked_entries.end ())
+      const auto mbox = pair.first.mbox ();
+      if (s_checked_entries.find (mbox) != s_checked_entries.end ())
         {
           continue;
         }
@@ -410,10 +458,10 @@ Addressbook::check_o (Mail *mail)
         {
           log_debug ("%s:%s: failed to resolve contact for %s",
                      SRCNAME, __func__,
-                     anonstr (pair.first.c_str()));
+                     anonstr (mbox.c_str()));
           continue;
         }
-      s_checked_entries.insert (pair.first);
+      s_checked_entries.insert (mbox);
 
       LPDISPATCH user_props = get_oom_object (contact.get (), "UserProperties");
       if (!user_props)
@@ -439,7 +487,7 @@ Addressbook::check_o (Mail *mail)
 
       log_debug ("%s:%s: found configured key for %s",
                  SRCNAME, __func__,
-                 anonstr (pair.first.c_str()));
+                 anonstr (mbox.c_str()));
 
       char *pgp_data = get_oom_string (pgp_key, "Value");
       char *cms_data = nullptr;
@@ -456,12 +504,12 @@ Addressbook::check_o (Mail *mail)
         }
       if (pgp_data && strlen (pgp_data))
         {
-          KeyCache::instance ()->importFromAddrBook (pair.first, pgp_data,
+          KeyCache::instance ()->importFromAddrBook (mbox, pgp_data,
                                                      mail, GpgME::OpenPGP);
         }
       if (cms_data && strlen (cms_data))
         {
-          KeyCache::instance ()->importFromAddrBook (pair.first, cms_data,
+          KeyCache::instance ()->importFromAddrBook (mbox, cms_data,
                                                      mail, GpgME::CMS);
         }
       xfree (pgp_data);
