@@ -30,6 +30,8 @@
 #include "overlay.h"
 #include "keycache.h"
 #include "mymapitags.h"
+#include "recipient.h"
+#include "windowmessages.h"
 
 #include <gpgme++/context.h>
 #include <gpgme++/signingresult.h>
@@ -70,7 +72,9 @@ CryptController::CryptController (Mail *mail, bool encrypt, bool sign,
   memdbg_ctor ("CryptController");
   log_debug ("%s:%s: CryptController ctor for %p encrypt %i sign %i inline %i.",
              SRCNAME, __func__, mail, encrypt, sign, mail->getDoPGPInline ());
-  m_recipient_addrs = mail->getCachedRecipients ();
+  m_recipients = mail->getCachedRecipients ();
+  m_signer_keys = mail->getSigningKeys ();
+  m_sender = mail->getSender ();
   TRETURN;
 }
 
@@ -184,8 +188,8 @@ CryptController::collect_data ()
 }
 
 int
-CryptController::lookup_fingerprints (const std::string &sigFpr,
-                                      const std::vector<std::string> recpFprs)
+CryptController::lookup_fingerprints (const std::vector<std::string> &sigFprs,
+                                      const std::vector<std::pair <std::string, std::string> > &recpFprs)
 {
   TSTART;
   auto ctx = std::shared_ptr<GpgME::Context> (GpgME::Context::createForProtocol (m_proto));
@@ -203,44 +207,112 @@ CryptController::lookup_fingerprints (const std::string &sigFpr,
   ctx->setKeyListMode (GpgME::Local);
   GpgME::Error err;
 
-  if (!sigFpr.empty()) {
-      m_signer_key = ctx->key (sigFpr.c_str (), err, true);
-      if (err || m_signer_key.isNull ()) {
+  if (sigFprs.size())
+    {
+      char **cSigners = vector_to_cArray (sigFprs);
+      err = ctx->startKeyListing (const_cast<const char **> (cSigners), true);
+      if (err)
+        {
+          log_error ("%s:%s: failed to start signer keylisting",
+                     SRCNAME, __func__);
+          release_cArray (cSigners);
+          TRETURN -1;
+        }
+      do
+        {
+          m_signer_keys.push_back(ctx->nextKey(err));
+        }
+      while (!err);
+
+      release_cArray (cSigners);
+
+      m_signer_keys.pop_back();
+
+      if (m_signer_keys.empty())
+        {
           log_error ("%s:%s: failed to lookup key for '%s' with protocol '%s'",
-                     SRCNAME, __func__, anonstr (sigFpr.c_str ()),
+                     SRCNAME, __func__, anonstr (sigFprs[0].c_str ()),
                      m_proto == GpgME::CMS ? "smime" :
                      m_proto == GpgME::OpenPGP ? "openpgp" :
                      "unknown");
           TRETURN -1;
-      }
+        }
       // reset context
       ctx = std::shared_ptr<GpgME::Context> (GpgME::Context::createForProtocol (m_proto));
       ctx->setKeyListMode (GpgME::Local);
-  }
+    }
 
-  if (!recpFprs.size()) {
-     TRETURN 0;
-  }
+  if (!recpFprs.size())
+    {
+      TRETURN 0;
+    }
 
-  // Convert recipient fingerprints
-  char **cRecps = vector_to_cArray (recpFprs);
+  std::vector<std::string> all_fingerprints;
+  for (const auto &pair: recpFprs)
+    {
+      all_fingerprints.push_back (pair.second);
+    }
 
-  err = ctx->startKeyListing (const_cast<const char **> (cRecps));
+  for (auto &recp: m_recipients)
+    {
+      std::vector <std::string> fingerprintsToLookup;
+      /* Find the matching fingerprints for the recpient. */
+      for (const auto &pair: recpFprs)
+        {
+          if (pair.first.empty())
+            {
+              log_error ("%s:%s Recipient mbox is empty. Wrong Resolver Version?",
+                         SRCNAME, __func__);
+            }
+          /* We also accept the empty string here for backwards compatibility
+             but we should still log an error to be clear. */
+          if (pair.first.empty() || pair.first == recp.mbox ())
+            {
+              fingerprintsToLookup.push_back (pair.second);
+              all_fingerprints.erase(std::remove_if(all_fingerprints.begin(),
+                                                    all_fingerprints.end(),
+                              [&pair](const std::string &x){return x == pair.second;}),
+                              all_fingerprints.end());
+            }
+        }
+      // Convert recipient fingerprints
+      char **cRecps = vector_to_cArray (fingerprintsToLookup);
+      err = ctx->startKeyListing (const_cast<const char **> (cRecps));
+      if (err)
+        {
+          log_error ("%s:%s: failed to start recipient keylisting",
+                     SRCNAME, __func__);
+          release_cArray (cRecps);
+          TRETURN -1;
+        }
 
-  if (err) {
-      log_error ("%s:%s: failed to start recipient keylisting",
-                 SRCNAME, __func__);
+      std::vector <GpgME::Key> keys;
+      do {
+          keys.push_back(ctx->nextKey(err));
+      } while (!err);
+
+      keys.pop_back ();
       release_cArray (cRecps);
+      recp.setKeys (keys);
+      // reset context
+      ctx = std::shared_ptr<GpgME::Context> (GpgME::Context::createForProtocol (m_proto));
+      ctx->setKeyListMode (GpgME::Local);
+    }
+  if (!all_fingerprints.empty())
+    {
+      log_error ("%s:%s: BUG: Not all fingerprints could be matched "
+                 "to a recipient.",
+                 SRCNAME, __func__);
+      for (const auto &fpr: all_fingerprints)
+        {
+          log_debug ("%s:%s Failed to find: %s", SRCNAME, __func__,
+                     anonstr (fpr.c_str ()));
+        }
+      clear_keys ();
       TRETURN -1;
-  }
+    }
 
-  do {
-      m_recipients.push_back(ctx->nextKey(err));
-  } while (!err);
-
-  m_recipients.pop_back();
-
-  release_cArray (cRecps);
+  resolving_done ();
 
   TRETURN 0;
 }
@@ -250,12 +322,11 @@ int
 CryptController::parse_output (GpgME::Data &resolverOutput)
 {
   TSTART;
-  // Todo: Use Data::toString
   std::istringstream ss(resolverOutput.toString());
   std::string line;
 
-  std::string sigFpr;
-  std::vector<std::string> recpFprs;
+  std::vector<std::string> sigFprs;
+  std::vector<std::pair<std::string, std::string>> recpFprs;
   while (std::getline (ss, line))
     {
       rtrim (line);
@@ -277,34 +348,31 @@ CryptController::parse_output (GpgME::Data &resolverOutput)
       std::string what;
       std::string how;
       std::string fingerprint;
+      std::string mbox;
 
       std::getline (lss, what, ':');
       std::getline (lss, how, ':');
       std::getline (lss, fingerprint, ':');
+      std::getline (lss, mbox, ':');
 
       if (m_proto == GpgME::UnknownProtocol)
         {
+          /* TODO: Allow mixed */
           m_proto = (how == "smime") ? GpgME::CMS : GpgME::OpenPGP;
         }
 
       if (what == "sig")
         {
-          if (!sigFpr.empty ())
-            {
-              log_error ("%s:%s: multiple signing keys not supported",
-                         SRCNAME, __func__);
-
-            }
-          sigFpr = fingerprint;
+          sigFprs.push_back (fingerprint);
           continue;
         }
       if (what == "enc")
         {
-          recpFprs.push_back (fingerprint);
+          recpFprs.push_back(std::make_pair(mbox, fingerprint));
         }
     }
 
-  if (m_sign && sigFpr.empty())
+  if (m_sign && sigFprs.empty())
     {
       log_error ("%s:%s: Sign requested but no signing fingerprint - sending unsigned",
                  SRCNAME, __func__);
@@ -320,33 +388,175 @@ CryptController::parse_output (GpgME::Data &resolverOutput)
       TRETURN -2;
     }
 
-  TRETURN lookup_fingerprints (sigFpr, recpFprs);
+  TRETURN lookup_fingerprints (sigFprs, recpFprs);
 }
 
-static bool
-resolve_through_protocol (const GpgME::Protocol &proto, bool sign,
-                          bool encrypt, const std::string &sender,
-                          const std::vector<std::string> &recps,
-                          std::vector<GpgME::Key> &r_keys,
-                          GpgME::Key &r_sig)
+/* Combines all recipient keys for this operation
+   into a list and copys the resolved recipients
+   back. */
+void
+CryptController::resolving_done ()
 {
   TSTART;
-  bool sig_ok = true;
-  bool enc_ok = true;
+  m_enc_keys.clear ();
 
-  const auto cache = KeyCache::instance();
+  for (const auto &recp: m_recipients)
+    {
+      const auto &keys = recp.keys ();
+      m_enc_keys.insert (m_enc_keys.end (), keys.begin (), keys.end ());
+    }
+  /* Copy the resolved recipients back to the mail */
+  m_mail->setRecipients (m_recipients);
+  m_mail->setSigningKeys (m_signer_keys);
+  TRETURN;
+}
 
-  if (encrypt)
+bool
+CryptController::resolve_through_protocol (GpgME::Protocol proto)
+{
+  TSTART;
+
+  const auto cache = KeyCache::instance ();
+
+  if (m_encrypt)
     {
-      r_keys = cache->getEncryptionKeys(recps, proto);
-      enc_ok = !r_keys.empty();
+      for (auto &recp: m_recipients)
+        {
+          recp.setKeys(cache->getEncryptionKeys(recp.mbox (), proto));
+        }
     }
-  if (sign && enc_ok)
+  if (m_sign)
     {
-      r_sig = cache->getSigningKey (sender.c_str (), proto);
-      sig_ok = !r_sig.isNull();
+      if (m_sender.empty())
+        {
+          log_error ("%s:%s: Asked to sign but sender is empty.",
+                     SRCNAME, __func__);
+          return false;
+        }
+      const auto key = cache->getSigningKey (m_sender.c_str (), proto);
+      if (m_signer_keys.empty ())
+        {
+          m_signer_keys.push_back (key);
+        }
+      for (const auto &k: m_signer_keys)
+        {
+          if (k.protocol () != proto)
+            {
+              m_signer_keys.push_back (key);
+              break;
+            }
+        }
     }
-  TRETURN sig_ok && enc_ok;
+  TRETURN is_resolved ();
+}
+
+/* Check if we can be resolved by a single protocol
+   and return it. */
+GpgME::Protocol
+CryptController::get_resolved_protocol () const
+{
+  TSTART;
+  GpgME::Protocol ret = GpgME::UnknownProtocol;
+
+  bool hasOpenPGPSignKey = false;
+  bool hasSMIMESignKey = false;
+  if (m_sign)
+    {
+      for (const auto &sig_key: m_signer_keys)
+        {
+          hasOpenPGPSignKey |= (!sig_key.isNull() &&
+                                sig_key.protocol () == GpgME::OpenPGP);
+          hasSMIMESignKey |= (!sig_key.isNull() &&
+                              sig_key.protocol () == GpgME::CMS);
+        }
+    }
+
+  if (m_encrypt)
+    {
+      for (const auto &recp: m_recipients)
+        {
+          if (!recp.keys ().size () || recp.keys ()[0].isNull ())
+            {
+              /* No keys or the first key in the list is null. */
+              TRETURN GpgME::UnknownProtocol;
+            }
+          for (const auto &key: recp.keys())
+            {
+              if (key.protocol () == GpgME::OpenPGP &&
+                  (!m_sign || hasOpenPGPSignKey) &&
+                  (ret == GpgME::UnknownProtocol || ret == GpgME::OpenPGP))
+                {
+                  ret = GpgME::OpenPGP;
+                  continue;
+                }
+              if (key.protocol () == GpgME::CMS &&
+                  (!m_sign || hasSMIMESignKey) &&
+                  (ret == GpgME::UnknownProtocol || ret == GpgME::CMS))
+                {
+                  ret = GpgME::CMS;
+                  continue;
+                }
+              // Unresolvable
+              TRETURN GpgME::UnknownProtocol;
+            }
+        }
+    }
+  TRETURN ret;
+}
+
+
+/* Check that the crypt operation is resolved. This
+   supports combined S/MIME and OpenPGP operations. */
+bool
+CryptController::is_resolved () const
+{
+  /* Check that we have signing keys if necessary and at
+     least one encryption key for each recipient. */
+  bool hasOpenPGPSignKey = false;
+  bool hasSMIMESignKey = false;
+  if (m_sign)
+    {
+      for (const auto &sig_key: m_signer_keys)
+        {
+          hasOpenPGPSignKey |= (!sig_key.isNull() &&
+                                sig_key.protocol () == GpgME::OpenPGP);
+          hasSMIMESignKey |= (!sig_key.isNull() &&
+                              sig_key.protocol () == GpgME::CMS);
+        }
+    }
+
+  log_dbg ("Has OpenPGP Sig Key: %i SMIME: %i",
+           hasOpenPGPSignKey, hasSMIMESignKey);
+
+  if (m_encrypt)
+    {
+      Recipient::dump (m_recipients);
+      for (const auto &recp: m_recipients)
+        {
+          if (!recp.keys ().size () || recp.keys ()[0].isNull ())
+            {
+              /* No keys or the first key in the list is null. */
+              return false;
+            }
+          /* If we don't sign we need no more checks. */
+          if (!m_sign)
+            {
+              continue;
+            }
+          for (const auto &key: recp.keys())
+            {
+              if (key.protocol () == GpgME::OpenPGP && !hasOpenPGPSignKey)
+                {
+                  return false;
+                }
+              if (key.protocol () == GpgME::CMS && !hasSMIMESignKey)
+                {
+                  return false;
+                }
+            }
+        }
+    }
+  return true;
 }
 
 int
@@ -354,20 +564,11 @@ CryptController::resolve_keys_cached()
 {
   TSTART;
   // Prepare variables
-  const auto cached_sender = m_mail->getSender ();
-  auto recps = m_recipient_addrs;
-
-  if (m_encrypt)
-    {
-      recps.push_back (cached_sender);
-    }
-
+  const auto recps = m_mail->getCachedRecipientAddresses ();
   bool resolved = false;
   if (opt.enable_smime && opt.prefer_smime)
     {
-      resolved = resolve_through_protocol (GpgME::CMS, m_sign, m_encrypt,
-                                           cached_sender, recps, m_recipients,
-                                           m_signer_key);
+      resolved = resolve_through_protocol (GpgME::CMS);
       if (resolved)
         {
           log_debug ("%s:%s: Resolved with CMS due to preference.",
@@ -377,9 +578,7 @@ CryptController::resolve_keys_cached()
     }
   if (!resolved)
     {
-      resolved = resolve_through_protocol (GpgME::OpenPGP, m_sign, m_encrypt,
-                                           cached_sender, recps, m_recipients,
-                                           m_signer_key);
+      resolved = resolve_through_protocol (GpgME::OpenPGP);
       if (resolved)
         {
           log_debug ("%s:%s: Resolved with OpenPGP.",
@@ -389,9 +588,7 @@ CryptController::resolve_keys_cached()
     }
   if (!resolved && (opt.enable_smime && !opt.prefer_smime))
     {
-      resolved = resolve_through_protocol (GpgME::CMS, m_sign, m_encrypt,
-                                           cached_sender, recps, m_recipients,
-                                           m_signer_key);
+      resolved = resolve_through_protocol (GpgME::CMS);
       if (resolved)
         {
           log_debug ("%s:%s: Resolved with CMS as fallback.",
@@ -400,40 +597,68 @@ CryptController::resolve_keys_cached()
         }
     }
 
+  for (const auto &recp: m_recipients)
+    {
+      log_debug ("Enc Key for: '%s' Type: %i", anonstr (recp.mbox ().c_str ()),
+                                                        recp.type ());
+      for (const auto &key: recp.keys ())
+        {
+          log_debug ("%s:%s", to_cstr (key.protocol ()),
+                     anonstr (key.primaryFingerprint ()));
+        }
+    }
+  for (const auto &sig_key: m_signer_keys)
+    {
+      log_debug ("%s:%s: Signing key: %s:%s",
+                 SRCNAME, __func__,
+                 to_cstr (sig_key.protocol()),
+                 anonstr (sig_key.primaryFingerprint ()));
+    }
+
   if (!resolved)
     {
       log_debug ("%s:%s: Failed to resolve through cache",
                  SRCNAME, __func__);
-      m_recipients.clear();
-      m_signer_key = GpgME::Key();
+      m_enc_keys.clear ();
+      m_signer_keys.clear ();
       m_proto = GpgME::UnknownProtocol;
       TRETURN 1;
     }
 
-  if (!m_recipients.empty())
+  if (m_encrypt)
     {
       log_debug ("%s:%s: Encrypting with protocol %s to:",
                  SRCNAME, __func__, to_cstr (m_proto));
     }
-  for (const auto &key: m_recipients)
-    {
-      log_debug ("%s", anonstr (key.primaryFingerprint ()));
-    }
-  if (!m_signer_key.isNull())
-    {
-      log_debug ("%s:%s: Signing key: %s:%s",
-                 SRCNAME, __func__, anonstr (m_signer_key.primaryFingerprint ()),
-                 to_cstr (m_signer_key.protocol()));
-    }
   TRETURN 0;
 }
 
+void
+CryptController::clear_keys ()
+{
+  for (auto &recp: m_recipients)
+    {
+      recp.setKeys (std::vector<GpgME::Key> ());
+    }
+  m_signer_keys.clear ();
+}
+
 int
 CryptController::resolve_keys ()
 {
   TSTART;
 
-  m_recipients.clear();
+  m_proto = get_resolved_protocol ();
+  if (m_proto != GpgME::UnknownProtocol)
+    {
+      log_debug ("%s:%s: Already resolved by %s Not resolving again.",
+                 SRCNAME, __func__, to_cstr (m_proto));
+      start_crypto_overlay();
+      resolving_done ();
+      TRETURN 0;
+    }
+
+  m_enc_keys.clear();
 
   if (m_mail->isDraftEncrypt() && opt.draft_key)
     {
@@ -453,11 +678,11 @@ CryptController::resolve_keys ()
       log_debug ("%s:%s: resolved draft encryption key protocol is: %s",
                  SRCNAME, __func__, to_cstr (key.protocol()));
       m_proto = key.protocol ();
-      m_recipients.push_back (key);
+      m_enc_keys.push_back (key);
       TRETURN 0;
     }
 
-  if (!m_recipient_addrs.size())
+  if (!m_recipients.size())
     {
       /* Should not happen. But we add it for better bug reports. */
       const char *bugmsg = utf8_gettext ("Operation failed.\n\n"
@@ -479,6 +704,7 @@ CryptController::resolve_keys ()
       log_debug ("%s:%s: resolved keys through the cache",
                  SRCNAME, __func__);
       start_crypto_overlay();
+      resolving_done ();
       TRETURN 0;
     }
 
@@ -562,9 +788,9 @@ CryptController::resolve_keys ()
     {
       args.push_back (std::string ("--encrypt"));
       // Get the recipients that are cached from OOM
-      for (const auto &addr: m_recipient_addrs)
+      for (const auto &recp: m_recipients)
         {
-          const auto mbox = GpgME::UserID::addrSpecFromString (addr.c_str());
+          const auto mbox = recp.mbox ();
           auto overrides =
             KeyCache::instance ()->getOverrides (mbox, GpgME::OpenPGP);
           const auto cms_overrides =
@@ -678,6 +904,7 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
   WKSHelper::instance()->start_check (m_mail->getSender ());
 
   int ret = resolve_keys ();
+
   if (ret == -1)
     {
       //error
@@ -690,6 +917,37 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
       // Cancel
       TRETURN -2;
     }
+
+  /* If we need to send multiple emails we jump back from
+     here into the main event loop. Copy the mail object
+     and send it out mutiple times. */
+  if (opt.splitBCCMails)
+    {
+      bool foundOneNormalRecp = false;
+      bool foundOneBCCRecp = false;
+      for (const auto &recp: m_recipients)
+        {
+          if (foundOneBCCRecp && foundOneNormalRecp)
+            {
+              log_debug ("%s:%s: Have both BCC and normal recipients."
+                         " Need to send multiple mails.",
+                         SRCNAME, __func__);
+              do_in_ui_thread_async (SEND_MULTIPLE_MAILS, m_mail);
+              /* Cancel the crypto of this mail */
+              TRETURN -3;
+            }
+          if (recp.type() == Recipient::olCC ||
+              recp.type() == Recipient::olTo)
+            {
+              foundOneNormalRecp = true;
+            }
+          if (recp.type() == Recipient::olBCC)
+            {
+              foundOneBCCRecp = true;
+            }
+        }
+    }
+
   bool do_inline = m_mail->getDoPGPInline ();
 
   if (m_proto == GpgME::CMS && do_inline)
@@ -712,9 +970,15 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
                          utf8_gettext ("GpgOL"), MB_OK);
       TRETURN -1;
     }
-  if (!m_signer_key.isNull())
+  if (!m_signer_keys.empty ())
     {
-      ctx->addSigningKey (m_signer_key);
+      for (const auto &key: m_signer_keys)
+        {
+          if (key.protocol () == m_proto)
+            {
+              ctx->addSigningKey (key);
+            }
+        }
     }
 
   ctx->setTextMode (m_proto == GpgME::OpenPGP);
@@ -723,7 +987,7 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
   if (m_encrypt && m_sign && do_inline)
     {
       // Sign encrypt combined
-      const auto result_pair = ctx->signAndEncrypt (m_recipients,
+      const auto result_pair = ctx->signAndEncrypt (m_enc_keys,
                                                     do_inline ? m_bodyInput : m_input,
                                                     m_output,
                                                     GpgME::Context::AlwaysTrust);
@@ -805,7 +1069,7 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
       m_output = GpgME::Data ();
       m_input = GpgME::Data ();
       multipart.seek (0, SEEK_SET);
-      const auto encResult = ctx->encrypt (m_recipients, multipart,
+      const auto encResult = ctx->encrypt (m_enc_keys, multipart,
                                            m_output,
                                            GpgME::Context::AlwaysTrust);
       err = encResult.error();
@@ -832,7 +1096,7 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
     }
   else if (m_encrypt)
     {
-      const auto result = ctx->encrypt (m_recipients, do_inline ? m_bodyInput : m_input,
+      const auto result = ctx->encrypt (m_enc_keys, do_inline ? m_bodyInput : m_input,
                                         m_output,
                                         GpgME::Context::AlwaysTrust);
       err = result.error();
