@@ -714,6 +714,7 @@ GpgolRibbonExtender::GetIDsOfNames (REFIID riid, LPOLESTR *rgszNames,
       ID_MAPPER (L"getSigTip", ID_GET_SIG_TTIP)
       ID_MAPPER (L"launchDetails", ID_LAUNCH_CERT_DETAILS)
       ID_MAPPER (L"getIsDetailsEnabled", ID_GET_IS_DETAILS_ENABLED)
+      ID_MAPPER (L"getIsAddrBookEnabled", ID_GET_IS_ADDR_BOOK_ENABLED)
       ID_MAPPER (L"getIsCrypto", ID_GET_IS_CRYPTO_MAIL)
       ID_MAPPER (L"openContactKey", ID_CMD_OPEN_CONTACT_KEY)
       ID_MAPPER (L"overrideFileClose", ID_CMD_FILE_CLOSE)
@@ -799,6 +800,8 @@ GpgolRibbonExtender::Invoke (DISPID dispid, REFIID riid, LCID lcid,
         return launch_cert_details (parms->rgvarg[0].pdispVal);
       case ID_GET_IS_DETAILS_ENABLED:
         return get_is_details_enabled (parms->rgvarg[0].pdispVal, result);
+      case ID_GET_IS_ADDR_BOOK_ENABLED:
+        return get_is_addr_book_enabled (parms->rgvarg[0].pdispVal, result);
 
       case ID_ON_LOAD:
           {
@@ -890,6 +893,26 @@ GetCustomUI_MIME (BSTR RibbonID, BSTR * RibbonXml)
         " onLoad=\"ribbonLoaded\">"
         " <ribbon>"
         "   <tabs>"
+/*
+        "    <tab idMso=\"TabOptions\">"
+        "     <group idMso=\"GroupRightsManagement\">"
+        "         <toggleButton idMso=\"EncryptMessage\""
+        "                 label=\"%s\""
+        "                 screentip=\"%s\""
+        "                 supertip=\"%s\""
+        "                 getPressed=\"getSignEncryptPressedEx\""
+        "                 getImage=\"btnSignEncryptLarge\""
+        "                 onAction=\"encryptSignMimeEx\"""/>"
+        "         <toggleButton id=\"DigitallySignMessage\""
+        "                 getImage=\"btnSignSmall\""
+        "                 label=\"%s\""
+        "                 screentip=\"%s\""
+        "                 supertip=\"%s\""
+        "                 onAction=\"signMimeEx\""
+        "                 getPressed=\"getSignPressedEx\"/>"
+        "     </group>"
+        "    </tab>"
+*/
         "    <tab idMso=\"TabNewMailMessage\">"
         "     <group id=\"general\""
         "            label=\"%s\">"
@@ -926,7 +949,12 @@ GetCustomUI_MIME (BSTR RibbonID, BSTR * RibbonXml)
         "    </tab>"
         "   </tabs>"
         " </ribbon>"
-        "</customUI>", _("GpgOL"),
+        "</customUI>",
+        /*
+        _("Sign"), signTTip, signSTip,
+        _("Encrypt"), encryptTTip, encryptSTip,
+        */
+        _("GpgOL"),
         _("Secure"), secureTTip, secureSTip,
         _("Sign"), signTTip, signSTip,
         _("Encrypt"), encryptTTip, encryptSTip,
@@ -1000,6 +1028,25 @@ GetCustomUI_MIME (BSTR RibbonID, BSTR * RibbonXml)
         "       </dialogBoxLauncher>"
         "     </group>"
         "    </tab>"
+        "    <tab idMso=\"TabContacts\">"
+        "     <group id=\"contacts_gpgol_ex\""
+        "            label=\"%s\">"
+        "       <button id=\"idContactAddkeyEx\""
+        "               getImage=\"btnSignEncryptLarge\""
+        "               size=\"large\""
+        "               label=\"%s\""
+        "               screentip=\"%s\""
+        "               supertip=\"%s\""
+        "               getEnabled=\"getIsAddrBookEnabled\""
+        "               onAction=\"openContactKey\"/>"
+        "       <dialogBoxLauncher>"
+        "         <button id=\"optsBtn_contact_ex\""
+        "                 onAction=\"openOptions\""
+        "                 screentip=\"%s\""
+        "                 supertip=\"%s\"/>"
+        "       </dialogBoxLauncher>"
+        "     </group>"
+        "    </tab>"
         "   </tabs>"
         "   <contextualTabs>"
         "   <tabSet idMso=\"TabComposeTools\">"
@@ -1055,6 +1102,15 @@ GetCustomUI_MIME (BSTR RibbonID, BSTR * RibbonXml)
         _("GpgOL"),
         optsSTip,
         _("GpgOL"),
+        _("Settings"),
+        _(/* TRANSLATORS: Tooltip caption */
+          "Configure GpgOL keys and settings for this contact."),
+        _(/* TRANSLATORS: Tooltip content */
+          "Configure contact specific keys and settings to override "
+          "the default behavior for this contact."),
+        _("GpgOL Settings"),
+        optsSTip,
+        _("GpgOL"),
         _("Secure"), secureTTip, secureSTip,
         _("Sign"), signTTip, signSTip,
         _("Encrypt"), encryptTTip, encryptSTip,
@@ -1187,7 +1243,7 @@ GpgolAddin::get_instance ()
     }
   if (addin_instance->isShutdown ())
     {
-      log_error ("%s:%s: Get instance after shutdown",
+      log_warn ("%s:%s: Get instance after shutdown",
                  SRCNAME, __func__);
     }
   return addin_instance;
