@@ -194,6 +194,14 @@ EVENT_SINK_INVOKE(MailItemEvents)
                          SRCNAME, __func__);
               TBREAK;
             }
+          int old_flags = get_gpgol_draft_info_flags (message);
+          if (old_flags)
+            {
+              log_dbg ("Open of already flagged mail. "
+                       "Not overwriting with settings.");
+              gpgol_release (message);
+              TBREAK;
+            }
           if (opt.encrypt_default)
             {
               draft_flags = 1;
@@ -439,7 +447,14 @@ EVENT_SINK_INVOKE(MailItemEvents)
               log_debug ("%s:%s: Send event for crypto mail %p saving and starting.",
                          SRCNAME, __func__, m_mail);
 
-              m_mail->prepareCrypto_o ();
+              if (m_mail->prepareCrypto_o ())
+                {
+                  log_dbg ("Prepare crypto requested send abort.");
+                  *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
+                  /* Reset the crypter state  */
+                  m_mail->setCryptState (Mail::NoCryptMail);
+                  TBREAK;
+                }
               m_mail->setIsDraftEncrypt (false);
 
               // Save the Mail
@@ -462,6 +477,8 @@ EVENT_SINK_INVOKE(MailItemEvents)
                                  "Crypto failed or canceled.",
                                  SRCNAME, __func__, m_object, m_mail);
                       *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
+                      /* Reset the crypter state  */
+                      m_mail->setCryptState (Mail::NoCryptMail);
                       TBREAK;
                     }
                   // For inline response we can't trigger send programatically
@@ -610,6 +627,12 @@ EVENT_SINK_INVOKE(MailItemEvents)
             }
           TRETURN S_OK;
         }
+      case ReadComplete:
+        {
+          log_oom ("%s:%s: ReadComplete: %p",
+                         SRCNAME, __func__, m_mail);
+          TRETURN S_OK;
+        }
       case Write:
         {
           log_oom ("%s:%s: Write : %p",
@@ -642,6 +665,11 @@ EVENT_SINK_INVOKE(MailItemEvents)
                          SRCNAME, __func__);
 
               m_mail->removeCategories_o ();
+              if (g_mail_copy_triggerer)
+                {
+                  log_dbg ("Needs first after write for copied mail.");
+                  g_mail_copy_triggerer->splitCopyMailCallback (m_mail);
+                }
               TBREAK;
             }
 
@@ -893,12 +921,25 @@ EVENT_SINK_INVOKE(MailItemEvents)
               draft_flags += 2;
             }
           bool is_crypto_mail = m_mail->isCryptoMail ();
-
+          if (opt.reply_crypt && is_crypto_mail)
+            {
+              int crypto_flags = m_mail->getCryptoFlags ();
+              if (crypto_flags)
+                {
+                  if (opt.sign_default)
+                    {
+                      /* When default signing is on we also sign unsigned encrypted
+                       * mails. */
+                      crypto_flags |= 2;
+                      /* reply / forward flags override the setting flags. */
+                    }
+                  draft_flags = crypto_flags;
+                }
+            }
           /* If it is a crypto mail and the settings should not be taken
            * from the crypto mail and always encrypt / sign is on. Or
            * If it is not a crypto mail and we have automaticalls sign_encrypt. */
-          if ((is_crypto_mail && !opt.reply_crypt && draft_flags) ||
-              (!is_crypto_mail && draft_flags))
+          if (draft_flags)
             {
               /* Check if we can use the dispval */
                 if (parms->cArgs == 2 && parms->rgvarg[1].vt == (VT_DISPATCH) &&
@@ -963,7 +1004,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                   else
                     {
                       log_debug ("%s:%s: Reply in the same loop as empty "
-                                 "load Marking %p (item %p) as reply.",
+                                 "load treating %p (item %p) as reply.",
                                  SRCNAME, __func__, last_mail,
                                  last_mail->item ());
                     }
@@ -996,36 +1037,6 @@ EVENT_SINK_INVOKE(MailItemEvents)
               // We can now invalidate the last mail
               Mail::clearLastMail ();
             }
-
-          log_oom ("%s:%s: Reply Forward ReplyAll: %p",
-                         SRCNAME, __func__, m_mail);
-          if (!opt.reply_crypt)
-            {
-              TBREAK;
-            }
-          int crypto_flags = 0;
-          if (!(crypto_flags = m_mail->getCryptoFlags ()))
-            {
-              TBREAK;
-            }
-          if (parms->cArgs != 2 || parms->rgvarg[1].vt != (VT_DISPATCH) ||
-              parms->rgvarg[0].vt != (VT_BOOL | VT_BYREF))
-            {
-              /* This happens in the weird case */
-              log_debug ("%s:%s: Unexpected args %i %x %x named: %i",
-                         SRCNAME, __func__, parms->cArgs, parms->rgvarg[0].vt, parms->rgvarg[1].vt,
-                         parms->cNamedArgs);
-              TBREAK;
-            }
-          LPMESSAGE msg = get_oom_base_message (parms->rgvarg[1].pdispVal);
-          if (!msg)
-            {
-              log_debug ("%s:%s: Failed to get base message",
-                         SRCNAME, __func__);
-              TBREAK;
-            }
-          set_gpgol_draft_info_flags (msg, crypto_flags);
-          gpgol_release (msg);
           TBREAK;
         }
       case AttachmentRemove:
