@@ -4,17 +4,17 @@
  * Software engineering by Intevation GmbH
  *
  * This file is part of GpgOL.
- * 
+ *
  * GpgOL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * GpgOL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
@@ -22,6 +22,8 @@
 #ifndef MIMEMAKER_H
 #define MIMEMAKER_H
 
+#include "mapihelp.h"
+
 class Mail;
 #ifdef __cplusplus
 extern "C" {
@@ -92,6 +94,10 @@ void create_top_signing_header (char *buffer, size_t buflen, protocol_t protocol
 int write_string (sink_t sink, const char *text);
 int write_b64 (sink_t sink, const void *data, size_t datalen);
 
+/* Encode an input string according to rfc2047
+   caller needs to free result. */
+char *utf8_to_rfc2047b (const char *input);
+
 #ifdef __cplusplus
 }
 #endif
